
/* Chrysalide - Outil d'analyse de fichiers binaires
 * processor.h - prototypes pour la définition générique interne des architectures
 *
 * Copyright (C) 2008-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ARCH_PROCESSOR_INT_H
#define _ARCH_PROCESSOR_INT_H






/* TODO : nettoyer ! */

#include <stdint.h>
#include <sys/types.h>


#include "operand.h"        /* TODO: AsmSyntax ? */
#include "instruction.h"
#include "processor.h"





/* Taille des pré-allocations pour les instructions */
#define INSTR_ALLOC_BLOCK 100



/* Fournit un contexte propre au processeur d'une architecture. */
typedef GProcContext * (* get_processor_context_fc) (const GArchProcessor *);

/* Fournit un contexte lié au processeur pour une décompilation. */
typedef GDecContext * (* get_decomp_context_fc) (const GArchProcessor *);

/* Décode une instruction dans un flux de données. */
typedef GArchInstruction * (* decode_instruction_fc) (const GArchProcessor *, GProcContext *, const bin_t *, off_t *, off_t, vmpa_t, GBinFormat *);

/* Désassemble une instruction dans un flux de données. */
typedef GArchInstruction * (* disass_instr_fc) (const GArchProcessor *, GProcContext *, const GBinContent *, vmpa2t *, GExeFormat *);



/* Couverture d'un groupe d'instructions */
struct _instr_coverage
{
    mrange_t range;                         /* Couverture du groupement    */

    size_t start;                           /* Indice de départ            */
    size_t count;                           /* Quantité d'inclusions       */

};




/* Définition générique d'un processeur d'architecture (instance) */
struct _GArchProcessor
{
    GObject parent;                         /* A laisser en premier        */

    SourceEndian endianness;                /* Boutisme de l'architecture  */
    MemoryDataSize memsize;                 /* Taille de l'espace mémoire  */
    MemoryDataSize inssize;                 /* Taille min. d'encodage      */

    get_processor_context_fc get_ctx;       /* Obtention d'un contexte #1  */
    get_decomp_context_fc get_dec_ctx;      /* Obtention d'un contexte #2  */

    GArchInstruction **instructions;        /* Instructions désassemblées  */
    size_t instr_allocated;                 /* Taille de la liste allouée  */
    size_t instr_count;                     /* Taille de la liste aplatie  */

    instr_coverage *coverages;              /* Liste de couvertures        */
    size_t cov_allocated;                   /* Taille de la liste allouée  */
    size_t cov_count;                       /* Taille de la liste utilisée */

};







/* Définition générique d'un processeur d'architecture (classe) */
struct _GArchProcessorClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    decode_instruction_fc decode;           /* Traduction en instructions  */
    disass_instr_fc disassemble;            /* Traduction en instructions  */

};




#define SKIPPED_INSTR ((void *)-1)






#endif  /* _ARCH_PROCESSOR_INT_H */
